<?PHP
/**
 * Describes a suggestion / warning for possible invalid legal statuses of an object's
 * or its representations legal status.
 *
 * @author Joshua Ramon Enslin <joshua@museum-digital.de>
 */
declare(strict_types = 1);

/**
 * Represents a possible status for to-dos.
 */
enum MDPlausiLegalCheckResultType implements JsonSerializable {

    case expect_public_domain;
    case expect_restricted_legal_status;

    /**
     * Returns a value of this type based on a string.
     *
     * @param string $input Input to get a value from.
     *
     * @return MDPlausiLegalCheckResultType
     */
    public static function fromString(string $input):MDPlausiLegalCheckResultType {

        return match($input) {
            'expect_public_domain' => self::expect_public_domain,
            'expect_restricted_legal_status' => self::expect_restricted_legal_status,
            default => throw new MDpageParameterNotFromListException("Unknown result type"),
        };

    }

    /**
     * Returns a value of this type based on a integer.
     *
     * @param integer $input Input to get a value from.
     *
     * @return MDPlausiLegalCheckResultType
     */
    public static function fromInt(int $input):MDPlausiLegalCheckResultType {

        return match($input) {
            1 => self::expect_public_domain,
            2 => self::expect_restricted_legal_status,
            default => throw new MDpageParameterNotFromListException("Unknown result type"),
        };

    }

    /**
     * Returns a numeric representation of the type, e.g. for efficient storage in a DB.
     *
     * @return integer
     */
    public function toInt():int {

        return match($this) {
            self::expect_public_domain => 1,
            self::expect_restricted_legal_status => 2,
        };

    }


    /**
     * Lists all available names.
     *
     * @return array<string>
     */
    public static function caseNames():array {

        $output = [];

        $cases = self::cases();
        foreach ($cases as $case) {
            $output[] = $case->name;
        }

        return $output;

    }

    /**
     * Returns a translation for the entry.
     *
     * @param MDTlLoader $tlLoader Translation loader.
     *
     * @return string
     */
    public function getTl(MDTlLoader $tlLoader):string {

        return $tlLoader->tl("quality", "quality", "legal_check_msg_type_" . $this->name);

    }

    /**
     * Provides the option to serialize as a string during json_encode().
     *
     * @return string
     */
    public function jsonSerialize():string {

        return $this->name;

    }
}
